// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace TextEditor { class BaseHoverHandler; }

namespace QmakeProjectManager::Internal {

TextEditor::BaseHoverHandler &proFileHoverHandler();

} // namespace QmakeProjectManager::Internal
