// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QString>

namespace Vcpkg::Internal {

QByteArray addDependencyToManifest(const QByteArray &manifest, const QString &package);

void setupVcpkgManifestEditor();

} // namespace Vcpkg::Internal
