// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace Utils::Constants {

const char FORM_MIMETYPE[] = "application/x-designer";
const char GLSL_MIMETYPE[] = "application/x-glsl";
const char JSON_MIMETYPE[] = "application/json";
const char JS_MIMETYPE[] = "application/javascript";
const char OCTET_STREAM_MIMETYPE[] = "application/octet-stream";
const char QBS_MIMETYPE[] = "application/x-qt.qbs+qml";
const char QMLPROJECT_MIMETYPE[] = "application/x-qmlproject";
const char QMLTYPES_MIMETYPE[] = "application/x-qt.meta-info+qml";
const char QMLUI_MIMETYPE[] = "application/x-qt.ui+qml";
const char SCXML_MIMETYPE[] = "application/scxml+xml";

const char AMBIGUOUS_HEADER_MIMETYPE[] = "application/vnd.qtc.ambiguousheader"; // not a real MIME type
const char PROFILE_MIMETYPE[] = "application/vnd.qt.qmakeprofile";
const char PROINCLUDEFILE_MIMETYPE [] = "application/vnd.qt.qmakeproincludefile";
const char PROFEATUREFILE_MIMETYPE [] = "application/vnd.qt.qmakeprofeaturefile";
const char PROCONFIGURATIONFILE_MIMETYPE [] = "application/vnd.qt.qmakeproconfigurationfile";
const char PROCACHEFILE_MIMETYPE [] = "application/vnd.qt.qmakeprocachefile";
const char PROSTASHFILE_MIMETYPE [] = "application/vnd.qt.qmakeprostashfile";
const char RESOURCE_MIMETYPE[] = "application/vnd.qt.xml.resource";

const char CMAKE_MIMETYPE[] = "text/x-cmake";
const char CMAKE_PROJECT_MIMETYPE[] = "text/x-cmake-project";
const char CPP_HEADER_MIMETYPE[] = "text/x-c++hdr";
const char CPP_SOURCE_MIMETYPE[] = "text/x-c++src";
const char CUDA_SOURCE_MIMETYPE[] = "text/vnd.nvidia.cuda.csrc";
const char C_HEADER_MIMETYPE[] = "text/x-chdr";
const char C_SOURCE_MIMETYPE[] = "text/x-csrc";
const char GLSL_FRAG_MIMETYPE[] = "text/x-glsl-frag";
const char GLSL_ES_FRAG_MIMETYPE[] = "text/x-glsl-es-frag";
const char GLSL_VERT_MIMETYPE[] = "text/x-glsl-vert";
const char GLSL_ES_VERT_MIMETYPE[] = "text/x-glsl-es-vert";
const char GLSL_GEOM_MIMETYPE[] = "text/x-glsl-es-geometry";
const char GLSL_COMP_MIMETYPE[] = "text/x-glsl-compute";
const char GLSL_TESS_MIMETYPE[] = "text/x-glsl-tessellation";
const char JAVA_MIMETYPE[] = "text/x-java";
const char LINGUIST_MIMETYPE[] = "text/vnd.qt.linguist";
const char MAKEFILE_MIMETYPE[] = "text/x-makefile";
const char MOC_MIMETYPE[] = "text/x-moc";
const char OBJECTIVE_CPP_SOURCE_MIMETYPE[] = "text/x-objc++src";
const char OBJECTIVE_C_SOURCE_MIMETYPE[] = "text/x-objcsrc";
const char QDOC_MIMETYPE[] = "text/x-qdoc";
const char QML_MIMETYPE[] = "text/x-qml";

} // Utils::Constants
