// Copyright (C) 2023 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace Copilot::Constants {
const char COPILOT_PROJECT_SETTINGS_ID[] = "Copilot.Project.Settings";
const char ENABLE_COPILOT[] = "Copilot.EnableCopilot";
const char COPILOT_USE_GLOBAL_SETTINGS[] = "Copilot.UseGlobalSettings";

const char COPILOT_TOGGLE[] = "Copilot.Toggle";
const char COPILOT_ENABLE[] = "Copilot.Enable";
const char COPILOT_DISABLE[] = "Copilot.Disable";
const char COPILOT_REQUEST_SUGGESTION[] = "Copilot.RequestSuggestion";
const char COPILOT_NEXT_SUGGESTION[] = "Copilot.NextSuggestion";
const char COPILOT_PREVIOUS_SUGGESTION[] = "Copilot.PreviousSuggestion";

const char COPILOT_GENERAL_OPTIONS_ID[] = "AI.Copilot";

} // namespace Copilot::Constants
