// Copyright (C) 2021 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace Conan::Constants {

const char INSTALL_STEP[]  = "ConanPackageManager.InstallStep";

} // Conan::Constants
