// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace Coco {
namespace Constants {

const char ACTION_ID[] = "coco.Action";
const char MENU_ID[] = "coco.Menu";
const char COCO_STEP_ID[] = "Cocoplugin.BuildStep";

const char PROFILE_NAME[] = "cocoplugin"; // Name of the Coco profile file

const char COCO_SETTINGS_GROUP[] = "Coco";
const char COCO_SETTINGS_PAGE_ID[] = "A.CocoOptions";

// Project settings
const char SETTINGS_NAME_KEY[] = "CocoProjectSettings";
const char SELECTION_DIR_KEY[] = "SelectionDir";
const char COCO_DIR_KEY[] = "CocoDirectory";
const char CSMES_PATH_KEY[] = "CsmesPath";

} // namespace Constants
} // namespace Coco
