// Copyright (C) 2018 Sergey Morozov
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace Cppcheck::Constants {

const char TEXTMARK_CATEGORY_ID[] = "Cppcheck";

const char OPTIONS_PAGE_ID[] = "Analyzer.Cppcheck.Settings";

const char CHECK_PROGRESS_ID[] = "Cppcheck.CheckingTask";

const char MANUAL_CHECK_PROGRESS_ID[] = "Cppcheck.ManualCheckingTask";
const char MANUAL_RUN_ACTION[] = "Cppcheck.ManualRun";
const char PERSPECTIVE_ID[] = "Cppcheck.Perspective";

} // Cppcheck::Constants
