// Copyright (C) 2025 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace DevContainer::Constants {
const char DEVCONTAINER_DEVICE_TYPE[] = "DevContainerDeviceType";
const char16_t DEVCONTAINER_FS_SCHEME[] = u"devcontainer";

const char ACTION_START_DEVCONTAINER[] = "devcontainer.start";
} // namespace DevContainer::Constants
